from .base import BaseAPI, ParentMethods
from . import filters


class community(BaseAPI, ParentMethods):
    """ Craigslist community API wrapper. """

    category = "ccc"
    filters = filters.query.community

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class act(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ats(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class kid(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class com(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class grp(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vnn(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class laf(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mis(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class muc(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pet(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pol(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rid(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vol(BaseAPI):
        filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class events(BaseAPI, ParentMethods):
    """ Craigslist events API wrapper. """

    category = "eee"
    filters = filters.query.events


class forsale(BaseAPI, ParentMethods):
    """ Craigslist for sale API wrapper. """

    category = "sss"
    filters = filters.query.forsale

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class ata(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ppa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ara(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sna(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pta(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wta(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ava(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class baa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bar(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bip(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bia(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bpa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class boo(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bka(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bfa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cta(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ema(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class moa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cla(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cba(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class syp(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sya(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ela(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class gra(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class zip(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fua(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class gms(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class foa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class haa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hva(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hsa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class jwa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class maa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mpa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mca(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class msa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pha(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rva(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sga(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tia(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tla(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class taa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tra(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vga(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class waa(BaseAPI):
        filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class gigs(BaseAPI, ParentMethods):

    category = "ggg"
    filters = filters.query.gigs

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class cpg(BaseAPI):
        filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class crg(BaseAPI):
        filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cwg(BaseAPI):
        filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class dmg(BaseAPI):
        filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class evg(BaseAPI):
        filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lbg(BaseAPI):
        filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tlg(BaseAPI):
        filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wrg(BaseAPI):
        filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class housing(BaseAPI, ParentMethods):
    """ Craigslist housing API wrapper. """

    category = "hhh"
    filters = filters.query.housing

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class apa(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class off(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class swp(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class prk(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rea(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class roo(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sub(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vac(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hou(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rew(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sha(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sbw(BaseAPI):
        filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class jobs(BaseAPI, ParentMethods):
    """ Craigslist jobs API wrapper. """

    category = "jjj"
    filters = filters.query.jobs

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class acc(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ofc(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class egr(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class med(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bus(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class csr(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class edu(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class etc(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fbh(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lab(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class gov(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hea(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hum(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lgl(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mnu(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mar(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class npo(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rej(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ret(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sls(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class spa(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sci(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sec(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class trd(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sof(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sad(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tch(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class trp(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tfr(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class web(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wri(BaseAPI):
        filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class resumes(BaseAPI, ParentMethods):
    """ Craigslist resumes API wrapper. """

    category = "rrr"
    filters = filters.query.resumes

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class services(BaseAPI, ParentMethods):
    """ Craigslist services API wrapper. """

    category = "bbb"
    filters = filters.query.services

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class aos(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bts(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cms(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cps(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class crs(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cys(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class evs(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fgs(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fns(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hss(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lbs(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lgs(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lss(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mas(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pas(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rts(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sks(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class biz(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class trv(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wet(BaseAPI):
        filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)



base = {
    "query": {"url_key": "query", "value": None},
    "search_titles": {"url_key": "srchType", "value": "T"},
    "has_image": {"url_key": "hasPic", "value": 1},
    "posted_today": {"url_key": "postedToday", "value": 1},
    "bundle_duplicates": {"url_key": "bundleDuplicates", "value": 1},
    "search_distance": {"url_key": "search_distance", "value": None},
    "zip_code": {"url_key": "postal", "value": None},
}

community = {
    **base,
    **{
        # no unique filters for community
    },
}

events = {
    **base,
    **{
        # art/film
        "art": {"url_key": "event_art", "value": 1, "attr": "art/film"},
        "film": {"url_key": "event_art", "value": 1, "attr": "art/film"},
        # career
        "career": {"url_key": "event_career", "value": 1, "attr": "career"},
        # charitable
        "charitable": {"url_key": "event_fundraiser_vol", "value": 1, "attr": "charitable"},
        "fundraiser": {"url_key": "event_fundraiser_vol", "value": 1, "attr": "charitable"},
        # competiton
        "athletics": {"url_key": "event_athletics", "value": 1, "attr": "competition"},
        "competition": {"url_key": "event_athletics", "value": 1, "attr": "competition"},
        # dance
        "dance": {"url_key": "event_dance", "value": 1, "attr": "dance"},
        # fest/fair
        "festival": {"url_key": "event_festival", "value": 1, "attr": "fest/fair"},
        "fair": {"url_key": "event_festival", "value": 1, "attr": "fest/fair"},
        # fitness/health
        "fitness": {"url_key": "event_fitness_wellness", "value": 1, "attr": "fitness/health"},
        "health": {"url_key": "event_fitness_wellness", "value": 1, "attr": "fitness/health"},
        # food/drink
        "food": {"url_key": "event_food", "value": 1, "attr": "food/drink"},
        "drink": {"url_key": "event_food", "value": 1, "attr": "food/drink"},
        # free
        "free": {"url_key": "event_free", "value": 1, "attr": "free"},
        # kid friendly
        "kid_friendly": {"url_key": "event_kidfriendly", "value": 1, "attr": "kid friendly"},
        # literary
        "literary": {"url_key": "event_literary", "value": 1, "attr": "literary"},
        # music
        "music": {"url_key": "event_music", "value": 1, "attr": "music"},
        # outdoor
        "outdoor": {"url_key": "event_outdoor", "value": 1, "attr": "outdoor"},
        # sale
        "sale": {"url_key": "event_sale", "value": 1, "attr": "sale"},
        # singles
        "singles": {"url_key": "event_singles", "value": 1, "attr": "singles"},
        # tech
        "tech": {"url_key": "event_geek", "value": 1, "attr": "tech"},
    },
}

forsale = {
    **base,
    **{
        # price
        "min_price": {"url_key": "min_price", "value": None},
        "max_price": {"url_key": "max_price", "value": None},
        # make and model
        "make": {"url_key": "auto_make_model", "value": None},
        "model": {"url_key": "auto_make_model", "value": None},
        # model year
        "min_year": {"url_key": "min_auto_year", "value": None},
        "max_year": {"url_key": "max_auto_year", "value": None},
        # odometer
        "min_miles": {"url_key": "min_auto_miles", "value": None},
        "max_miles": {"url_key": "max_auto_miles", "value": None},
        # engine displacement (cc)
        "min_engine_displacement": {"url_key": "min_engine_displacement_cc", "value": None},
        "max_engine_displacement": {"url_key": "max_engine_displacement_cc", "value": None},
    },
}

gigs = {
    **base,
    **{
        # paid/unpaid
        "is_paid": {"url_key": "is_paid", "value": None},
    },
}

housing = {
    **base,
    **{
        # price
        "min_price": {"url_key": "min_price", "value": None},
        "max_price": {"url_key": "max_price", "value": None},
        # bedrooms
        "min_bedrooms": {"url_key": "min_bedrooms", "value": None},
        "max_bedrooms": {"url_key": "max_bedrooms", "value": None},
        # bathrooms
        "min_bathrooms": {"url_key": "min_bathrooms", "value": None},
        "max_bathrooms": {"url_key": "max_bathrooms", "value": None},
        # ft2
        "min_ft2": {"url_key": "minSqft", "value": None},
        "max_ft2": {"url_key": "maxSqft", "value": None},
        # private room
        "private_room": {"url_key": "private_room", "value": 1, "attr": "private room"},
        # private bath
        "private_bath": {"url_key": "private_bath", "value": 1, "attr": "private bath"},
        # cats ok
        "cats_ok": {"url_key": "pets_cat", "value": 1, "attr": "cats are ok - purrr"},
        # dogs ok
        "dogs_ok": {"url_key": "pets_dog", "value": 1, "attr": "dogs are ok - wooof"},
        # furnished
        "is_furnished": {"url_key": "is_furnished", "value": 1, "attr": "furnished"},
        # no smoking
        "no_smoking": {"url_key": "no_smoking", "value": 1, "attr": "no smoking"},
        # wheelchair access
        "wheelchair_acccess": {
            "url_key": "wheelchaccess",
            "value": 1,
            "attr": "wheelchair accessible",
        },
        # EV charging
        "ev_charging": {"url_key": "ev_charging", "value": 1, "attr": "ev charging"},
        # no appliation fee
        "no_application_fee": {"url_key": "application_fee", "value": 1},
        # no broker fee
        "no_broker_fee": {"url_key": "broker_fee", "value": 1},
    },
}

jobs = {
    **base,
    **{
        # internship
        "is_internship": {"url_key": "is_internship", "value": 1, "attr": "internship"},
        # non-profit
        "is_nonprofit": {"url_key": "is_nonprofit", "value": 1, "attr": "non-profit organization"},
        # telecommute
        "is_telecommuting": {
            "url_key": "is_telecommuting",
            "value": 1,
            "attr": "telecommuting okay",
        },
    },
}

resumes = {
    **base,
    **{
        # TODO: Please create an issue or PR if interested in this category.
    },
}

services = {
    **base,
    **{
        # no unique filters for services
    },
}



from . import category, query

category = {
    "community": category.community,
    "events": category.events,
    "forsale": category.forsale,
    "gigs": category.gigs,
    "housing": category.housing,
    "jobs": category.jobs,
    "resumes": category.resumes,
}



community = {
    "act": "activity partners",
    "ats": "artists",
    "kid": "childcare",
    "com": "general community",
    "grp": "groups",
    "vnn": "local news and views",
    "laf": "lost & found",
    "mis": "missed connections",
    "muc": "musicians",
    "pet": "pets",
    "pol": "politics",
    "rnr": "rants & raves",
    "rid": "rideshare",
    "vol": "volunteers",
}

events = {}

forsale = {
    "ata": "antiques",
    "ppa": "appliances",
    "ara": "arts & crafts",
    "sna": "atvs, utvs, snowmobiles",
    "pta": "auto parts",
    "wta": "auto wheels & tires",
    "ava": "aviation",
    "baa": "baby & kid stuff",
    "bar": "barter",
    "bip": "bicycle parts",
    "bia": "bicycles",
    "bpa": "boat parts & accessories",
    "boo": "boats",
    "bka": "books & magazines",
    "bfa": "business",
    "cta": "cars & trucks",
    "ema": "cds / dvds / vhs",
    "moa": "cell phones",
    "cla": "clothing & accessories",
    "cba": "collectibles",
    "syp": "computer parts",
    "sya": "computers",
    "ela": "electronics",
    "gra": "farm & garden",
    "zip": "free stuff",
    "fua": "furniture",
    "gms": "garage & moving sales",
    "foa": "general for sale",
    "haa": "health and beauty",
    "hva": "heavy equipment",
    "hsa": "household items",
    "jwa": "jewelry",
    "maa": "materials",
    "mpa": "motorcycle parts & accessories",
    "mca": "motorcycles/scooters",
    "msa": "musical instruments",
    "pha": "photo/video",
    "rva": "recreational vehicles",
    "sga": "sporting goods",
    "tia": "tickets",
    "tla": "tools",
    "taa": "toys & games",
    "tra": "trailers",
    "vga": "video gaming",
    "waa": "wanted",
}

gigs = {
    "cpg": "computer gigs",
    "crg": "creative gigs",
    "cwg": "crew gigs",
    "dmg": "domestic gigs",
    "evg": "event gigs",
    "lbg": "labor gigs",
    "tlg": "talent gigs",
    "wrg": "writing gigs",
}

housing = {
    "apa": "apartments / housing for rent",
    "swp": "housing swap",
    "off": "office & commercial",
    "prk": "parking & storage",
    "rea": "real estate",
    "roo": "rooms & shares",
    "sub": "sublets & temporary",
    "vac": "vacation rentals",
    "hou": "wanted: apts",
    "rew": "wanted: real estate",
    "sha": "wanted: room/share",
    "sbw": "wanted: sublet/temp",
}

jobs = {
    "acc": "accounting/finance",
    "ofc": "admin/office",
    "egr": "architect/engineer/cad",
    "med": "art/media/design",
    "bus": "business/mgmt",
    "csr": "customer service",
    "edu": "education/teaching",
    "etc": "et cetera",
    "fbh": "food/beverage/hospitality",
    "lab": "general labor",
    "gov": "government",
    "hea": "healthcare",
    "hum": "human resource",
    "lgl": "legal/paralegal",
    "mnu": "manufacturing",
    "mar": "marketing/advertising/pr",
    "npo": "nonprofit",
    "rej": "real estate",
    "ret": "retail/wholesale",
    "sls": "sales",
    "spa": "salon/spa/fitness",
    "sci": "science/biotech",
    "sec": "security",
    "trd": "skilled trades/artisan",
    "sof": "software/qa/dba/etc",
    "sad": "systems/networking",
    "tch": "technical support",
    "trp": "transportation",
    "tfr": "tv/film/video/radio",
    "web": "web/html/info design",
    "wri": "writing/editing",
}

resumes = {}



from . import sessions


def get_addl_readable_filters(url):
    """ Gets additional Craigslist query filters in readable format. """
    filters = get_addl_filters(url)
    filters_iter = filters.copy()
    for key, value in filters_iter.items():
        if isinstance(value, dict):
            # get only the filter key bound to its numerical selector value
            filters[key] = list(value)

    return filters


def get_posts(url, filters, **kwargs):
    """ Gets every post on Craigslist as a dictionary with string values,
    provided a query url and filters. """
    # pass copy of filters to prevent manipulation by reference
    parsed_filters = parse_filters(url, filters.copy())
    # iterate through listings pages from a given search query
    search_html = sessions.get_html(url, params=parsed_filters)
    total_posts = get_total_posts(search_html)
    # there are 120 posts per listing page
    for search_page in range(int(total_posts / 120) + 1):
        search_page *= 120
        parsed_filters["s"] = search_page
        search_html = sessions.get_html(url, params=parsed_filters)

        yield from get_posts_in_search_page(search_html, **kwargs)


def parse_filters(url, filters):
    """ Takes query filters and parses them to match keyword value
    with appropriate integer value. """
    # TODO: can you include multiple filters per category?
    addl_filters = get_addl_filters(url)
    filters_iter = filters.copy()
    for key, value in filters_iter.items():
        if key in addl_filters:
            filters[key] = addl_filters[key][value]

    return filters


def get_addl_filters(url):
    """ Gets additional Craigslist query filters. """
    filters = {}
    search_html = sessions.get_html(url)
    for list_filter in search_html.find_all("div", class_="search-attribute"):
        filter_key = list_filter.attrs["data-attr"]
        filter_labels = list_filter.find_all("label")
        filters[filter_key] = {
            opt.text.strip(): opt.find("input").get("value") for opt in filter_labels
        }

    return filters


def get_total_posts(search_html_content):
    """ Gets total number of posts from Craigslist search page. """
    totalcount = search_html_content.find("span", {"class": "totalcount"})
    return int(totalcount.text) if totalcount else None


def get_posts_in_search_page(parsed_html, **kwargs):
    """ Yields posts content from Craigslist search HTML document
    given a page of Craigslist listings. """
    limit = kwargs["limit"]
    posts = parsed_html.find("ul", {"class": "rows"})
    for index, post in enumerate(posts.find_all("li", {"class": "result-row"}, recursive=False)):
        yield get_post_content(post, **kwargs)
        # search all listing if limit value is 0
        if index + 1 == limit:
            break


def get_post_content(post_html, **kwargs):
    """ Gets content from an individual post's HTML. """
    # prep bs4 html for parse
    header_link = post_html.find("a", {"class": "hdrlnk"})
    neighborhood = post_html.find("span", {"class": "result-hood"})
    price = post_html.find("span", {"class": "result-price"})
    repost_of = post_html.attrs.get("data-repost-of")
    time = post_html.find("time")
    if time:
        datetime = time.attrs["datetime"]
    else:
        pl = post_html.find("span", {"class": "pl"})
        datetime = pl.text.split(":")[0].strip() if pl else None

    # parse bs4 html
    post_id = post_html.attrs["data-pid"]
    post_repost_of = "" if not repost_of else repost_of
    post_last_updated = datetime
    post_title = header_link.text
    post_neighborhood = neighborhood.text.strip()[1:-1] if neighborhood else ""
    post_price = price.text if price else ""
    post_url = header_link.attrs["href"]
    post_site, post_area, post_category = parse_url_subdomains(post_url)

    # construct post content
    post_content = {
        "site": post_site,
        "area": post_area,
        "category": post_category,
        "id": post_id,
        "repost_of": post_repost_of,
        "last_updated": post_last_updated,
        "title": post_title,
        "neighborhood": post_neighborhood,
        "price": post_price,
        "url": post_url,
    }
    # if additional content is available, concat to post content
    post_content.update(get_addl_content(post_html, **kwargs))

    return post_content


def parse_url_subdomains(url):
    """ Parses site, area, and catgory from Craigslist query URL. """
    parsed_url = url.split("https://")[1].split(".")
    parsed_suburl = parsed_url[2].split("/")
    site = parsed_url[0]
    # parsed_suburls will have len == 5 if query has no area
    # len == 6 if query has area
    if len(parsed_suburl) == 5:
        area = ""
        category = parsed_suburl[1]
    else:
        area = parsed_suburl[1]
        category = parsed_suburl[2]

    return site, area, category


def get_addl_content(post_html, **kwargs):
    """ Gets additional post content from HTML document. """
    category = kwargs["category"]
    try:
        get_content = AddlContent.from_search_page[category]
        return get_content(post_html)
    except KeyError:
        return {}


# Though only containing one key and method, if a listings page is found to
# have extra post attributes, add appropriate selector and parsing method
# to this class.
class AddlContent:
    """ Gets additional post content from post's HTML and Craigslist
    search category. """

    from_search_page = {
        "apa": lambda post: AddlContent.parse_housing(
            post.find("span", {"class": "housing"}).contents
        )
        if post.find("span", {"class": "housing"})
        else ""
    }

    @staticmethod
    def parse_housing(contents):
        """ Parses additional content in apartments / housing for rent. """
        housing_attr = {}
        try:
            content = contents[0]
        except IndexError:
            content = []
        # this assumes no bedroom is > 99
        if "br" in content:
            housing_attr["bedrooms"] = str(
                int(content[content.find("br") - 2 : content.find("br")])
            )
        # for ft2 - assuming no area is greater than 10^10 ft2
        if "ft" in content:
            housing_attr["area-ft2"] = str(
                int(content[content.find("ft") - 10 : content.find("ft")])
            )
        # for m2 - assuming no area is greater than 10^10 m2
        if "m" in content:
            housing_attr["area-m2"] = str(int(content[content.find("m") - 10 : content.find("m")]))

        return housing_attr



import time
import requests
from requests.exceptions import RequestException
from bs4 import BeautifulSoup

USER_AGENT = "Mozilla/5.0"


def get_html(*args, **kwargs):
    """ Gets HTML document from requests content. """
    content = get_request(*args, **kwargs).content
    return BeautifulSoup(content, "html.parser")


def get_request(*args, max_retry=10, **kwargs):
    """ Gets requests.models.Response object using requests.get.
    Retry request if request fails, with number of iteration specified
    by max_retry. """
    kwargs.setdefault("headers", {}).setdefault("User-Agent", USER_AGENT)

    def recurse_get_request(*args, retries=0, wait=0.005, **kwargs):
        try:
            return requests.get(*args, **kwargs)
        except RequestException as exc:
            if retries > max_retry:
                raise RequestException from exc

            print("Request failed: (%s). Retrying after %s seconds ..." % (exc, "%.2f" % wait))
            time.sleep(wait)
            # double the wait time after every iteration
            wait *= 2
            retries += 1
            return recurse_get_request(*args, retries=retries, wait=wait, **kwargs)

    return recurse_get_request(*args, **kwargs)
